% LAT - Latvian DCE, 2017-01-03

clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('LAT_data.mat');
DATA = load(EstimOpt.DataFile);
load('IndComplete.mat');


%% ****************************  data transformations  ****************************


DATA.SQ = DATA.ALT == 3;

% unique(DATA.RS) % reduced no. of native species
DATA.RS0 = DATA.RS == 0; % baseline, no change
DATA.RS1 = DATA.RS == 1; % minor improvement
DATA.RS2 = DATA.RS == 2; % major improvement

% unique(DATA.WQ) % water quality
DATA.WQ0 = DATA.WQ == 0; % baseline, no change
DATA.WQ1 = DATA.WQ == 1; % minor improvement
DATA.WQ2 = DATA.WQ == 2; % major improvement

% unique(DATA.IS) % invasive species
DATA.IS0 = DATA.IS == 0; % baseline, no change
DATA.IS1 = DATA.IS == 1; % minor improvement
DATA.IS2 = DATA.IS == 2; % major improvement

DATA.inc(DATA.Q51 == 1) = 50;
DATA.inc(DATA.Q51 == 2) = 75;
DATA.inc(DATA.Q51 == 3) = 125;
DATA.inc(DATA.Q51 == 4) = 175;
DATA.inc(DATA.Q51 == 5) = 225;
DATA.inc(DATA.Q51 == 6) = 275;
DATA.inc(DATA.Q51 == 7) = 325;
DATA.inc(DATA.Q51 == 8) = 475;
DATA.inc(DATA.Q51 == 9) = 450;
DATA.inc(DATA.Q51 == 10) = 500;
DATA.inc = DATA.inc'*1.5/100;
DATA.inc_nan = DATA.Q51 == 11;


%% ****************************  model specification  ****************************


DATA.Y = DATA.CHOICE;

% recode all choices of those who are not willing to pay to SQ.
% DATA.Q23: Q23 'Would you personally be willing to pay anything to improve the Latvian marine environment quality concerning the described problems?'
% 1 'Yes' 2 'No' 3 'Not sure / Hard to say'
DATA.Y(DATA.SQ == 1 & DATA.Q23 == 2) = 1;
DATA.Y(DATA.SQ ~= 1 & DATA.Q23 == 2) = 0;

DATA.X = [DATA.SQ, DATA.RS1, DATA.RS2, DATA.WQ1, DATA.WQ2, DATA.IS1, DATA.IS2, -DATA.COST];
EstimOpt.NamesA = {'Status quo';'RS 1';'RS 2';'WQ 1';'WQ 2';'IS 1';'IS 2';'-Cost'};

DATA.Xstr = [DATA.Age, DATA.Q42==1, DATA.Q43==1, DATA.Q44, DATA.Q45, DATA.Q46==1, DATA.Q46==2, DATA.Q46==3, DATA.Q46==4,...
    DATA.Q47==2, DATA.Q47==3, DATA.Q47==4, DATA.Q47==5, DATA.Q47==6, DATA.Q47==7,...
    DATA.Q49_region==2, DATA.Q49_region==3, DATA.Q49_region==4, DATA.Q49_region==5, DATA.Q49_region==6, ...
    DATA.inc/100, DATA.inc_nan]; % Structural equations variables
EstimOpt.NamesStr = {'age'; 'male'; 'Latvian'; 'HH size'; 'HH children'; 'edu 1'; 'edu 2'; 'edu 3'; 'edu 4'; ...
    'Occ part'; 'Occ retired'; 'Occ student'; 'Occ home'; 'Occ self'; 'Occ unemployed';...
    'region - Pier?ga'; 'region - Vidzeme'; 'region - Kurzeme'; 'region - Zemgale'; 'region - Latgale';...
    'income'; 'income missing'};


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter); %(~(DATA.Q23 == 2));%(~(DATA.Design == 1 & DATA.ChT10_Q33==2)); %(DATA.CT == 1); %DATA.liczba_alt == 2);
INPUT.Xa = DATA.X(DATA.filter,:); %(~(DATA.Q23 == 2),:);%(~(DATA.Design == 1 & DATA.ChT10_Q33==2),:); %(DATA.CT == 1,:); %(DATA.liczba_alt == 2,:);
INPUT.Xstr = DATA.Xstr(DATA.filter,:);

MissingCompleteInd = MissingCompleteInd>0;
INPUT.MissingInd = MissingCompleteInd(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person
EstimOpt.NAlt = 3; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


% EstimOpt.eps = 1.e-9; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 5e3; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1; % maximum number of iterations

% OptimOpt.Algorithm = 'trust-region'; %'quasi-newton'
% EstimOpt.NumGrad = 0; % 1
% OptimOpt.GradObj = 'off'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% EstimOpt.HessEstFix = 1 ; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian


%% **************************** MNL ****************************


EstimOpt.WTP_space = 1; % number of cost parameters for WTP-space estimation (need to come last in Xa)

Results.MNL.b0 = [11.4383523792643;0.609697113452165;-0.555302503418268;6.08650518256211;7.03325410964056;1.16726353556797;1.88100985200941;0.101049993859052];

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     MXL     ****************************


EstimOpt.Dist = [1 0 0 0 0 0 0 0 1];

% WTP-space 10k: -7158.0878
Results.MXL_d.b0 = [15.1279823820561;0.729943078994710;0.490037034246966;3.92913369477579;5.16898493657720;0.766940526609503;0.508488564207601;-1.26659997615770;62.2662521098059;0.190979074045733;-0.988504197983243;2.27179957464709;4.51144722075429;0.673811692065140;0.314647349921147;0.915257951659733];

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     HMXL     ****************************


EstimOpt.NLatent = 6;

EstimOpt.Dist = EstimOpt.Dist(2:end);

% WTP-space 5k: -6518.3962
B_backup = [8.20583110911179;0.381799819233508;0.195752338927554;4.25180479609578;4.78670230094065;1.64287618533093;0.894508511192959;-0.108908088637944;55.2953819918908;0.415358066049843;-0.245886555177294;-0.0205183802457248;-0.0732094734697332;0.0937480473910764;0.383920593696897;0.117254355327568;0.0386163362927126;-0.303387172382379;0.752704828425848;3.68554270882584;4.74537145001350;1.07818511102640;1.73892836444202;0.601671310282094;39.8539962781360;2.14088514197268;2.52743631630725;-7.41077289317839;-11.7993643124919;-2.06696074572938;-1.38427435852413;2.62712746357635;28.2739392668423;-0.0841959304010255;0.0337601347232808;-3.64905178585583;-2.30496734984450;-2.68820645931585;-1.75918840113707;2.85469165843152;-24.4269212380562;0.691844981738359;2.50135279787381;8.24139701235422;13.1285473240498;1.90478052848217;1.62635756208044;-1.67640875731633;-0.625079971375999;0.0955929122398521;-0.0782735677674548;-1.56823412650690;-1.59408821147821;0.496099925339930;0.359451796185639;-0.163623909649737;16.6476682655571;-1.59506179038113;-3.65954563996019;-4.51524013862150;-5.35072252555185;-1.37308798760673;-1.99200322630269;0.616053879787501;0.281661776904487;0.0970205939145833;-0.251607091014492;0.472392714776428;-0.144587102844124;1.04422537923347;-0.304718409257064;-0.256805023948746;-0.320159966445069;0.0575205801266360;-0.00701040655890735;0.203436973832125;-0.00639926416277319;-0.0690234600430995;0.335127463655058;0.279358147868689;0.243456267643148;-0.200817487368986;0.0863409407786799;0.104654153102321;0.220776295709691;0.177317975492130;0.559836999079937;0.501215266308727;-1.29086063399570;1.18453186185057;-0.0990168608936484;6.12851340708348;-0.590466376909233;-1.28455950668009;-1.17443933098079;-0.284455149618724;0.411997264715832;0.788274499469566;0.228355180880016;-0.0269905612502517;1.30932183380175;0.799330594095959;1.26339730498692;-0.620277345465921;0.506597391263728;0.465306764863687;0.733001471815773;0.516378701882438;0.0461482852640020;0.0144636284073120;0.228811765739202;-0.409535441770143;0.202244420074202;-1.51491358278746;0.0849365513358591;0.219202066223425;0.253823931250234;0.0639185038939571;-0.0725199148843286;-0.0296607836525028;-0.0617451918460344;-0.0311247499743053;-0.315778619049025;-0.278553226590083;-0.181911206896869;-0.132479830695419;-0.113466525960905;-0.189727142910284;-0.300747801902361;-0.317796223100241;-0.133935900679160;0.287210214400587;-0.408257427696963;0.133480956128042;0.192180404048728;1.02541700055480;-0.210305775943527;-0.508533531425700;-0.369603044656253;-0.216990414543807;0.0177954543271972;0.456587518944893;0.0454927098352510;-0.0519903452432771;0.365532992133788;0.413721440116033;0.478282479584782;-0.0808916285486420;0.232649252018455;0.0446963568478134;0.202267771535412;0.106088497915483;1.10419358579587;0.184226405424206;0.636311134632119;0.173981185428697;0.407464108461906;0.0705808852578091;-0.163525854135507;-0.557992791719466;0.161254993366435;-0.0900261901779121;-1.01571968624397;0.828636368036622;-0.348121123330349;0.146907668375958;-0.227732392945899;-0.240849875971230;0.104554229478285;-0.914385404282020;-0.481144176977447;-0.233237924719318;-0.365558911677695;-0.330175099854265;0.500470244833405;0.362541668555859;-0.789931506743715;0.244407463920267;-0.0878539868351394;1.10901747420790;-0.234315726955095;-0.650454325509495;-0.465715913458835;-0.0372665439985826;0.00299239688504066;0.420764794031340;0.101551137902323;-0.00302391083094178;0.573053294458384;0.758376588491966;0.758475303278390;0.476891620617584;0.528672981002662;0.560940610069344;0.242008732225187;0.257043355985735];

Results.HMXL_d = HMXL(INPUT,Results,EstimOpt,OptimOpt);


%% Simulate WTP:

HHtypes = ...
    [20, 45, 35, 30, 70; ... % age
    0,  1,  1,  0,  0; ... % male
    1,  1,  0,  0,  1; ... % latvian
    1,  6,  2,  2,  1; ... % HHsize
    0,  4,  0,  1,  0; ... % children
    0,  0,  0,  0,  0; ... % edu primary
    0,  0,  0,  1,  1; ... % edu compulsory
    1,  0,  0,  0,  0; ... % edu general secondary
    0,  1,  0,  0,  0; ... % edu vocational secondary
    0,  0,  0,  0,  0; ... % 'Occ part'
    0,  0,  0,  0,  1; ... % 'Occ retired'
    1,  0,  0,  0,  0; ... % 'Occ student'
    0,  0,  0,  1,  0; ... % 'Occ home'
    0,  0,  1,  0,  0; ... % 'Occ self'
    0,  0,  0,  0,  0; ... % 'Occ unemployed'
    0,  0,  0,  1,  0; ... % 'region - Pier?ga'
    0,  1,  0,  0,  0; ... % 'region - Vidzeme'
    0,  0,  0,  0,  1; ... % 'region - Kurzeme'
    0,  0,  0,  0,  0; ... % 'region - Zemgale'
    0,  0,  0,  0,  0; ... % 'region - Latgale'
    0.7,4.1,7.1,  0,2.6; ... % inc
    0,  0,  0,  1,  0];    % inc missing

HHtypes(21,:) = HHtypes(21,:)./100;

HHtypes = HHtypes - mean(INPUT.Xstr)';
HHtypes = HHtypes ./ std(INPUT.Xstr)';

% load LL_hmxl_out_msLV.mat
sLV = [1.67099202659798;6.98288651283450;1.93589964551027;1.81505132801481;1.96220288437315;2.07394359131122];
mLV = [1.81136856547586e-07;6.39860214790653e-07;-2.03074921290098e-07;-1.29260042720640e-07;2.88424332253742e-08;-1.67610546251204e-07];

sim1 = 1e5;
sim2 = 1e5;
NVarA = 8;

bhat = mvnrnd(Results.HMXL_d.bhat,Results.HMXL_d.ihess,sim1)';
WTP_i = zeros(NVarA-1,size(HHtypes,2),sim1);

parfor i = 1:sim1
    bhat_i = bhat(:,i);
    LV = HHtypes'*reshape(bhat_i(NVarA*(2+EstimOpt.NLatent)+1:end),[size(HHtypes,1),EstimOpt.NLatent]);
    LV = LV - mLV';
    LV = LV ./ sLV';
    B0_i = mvnrnd(bhat_i(1:NVarA),diag(bhat_i(NVarA+1:NVarA*2).^2),sim2)';
    B0_i = reshape(B0_i,[NVarA,1,sim2]) + reshape(bhat_i(NVarA*(2)+1:NVarA*(2+EstimOpt.NLatent)),[NVarA,EstimOpt.NLatent])*LV';
    WTP_i(:,:,i) = mean(B0_i(1:NVarA-1,:,:),3);
end

Wm = median(WTP_i,3);
Ws = std(WTP_i,[],3);
Wq1 = quantile(WTP_i,0.025,3);
Wq2 = quantile(WTP_i,0.975,3);

RES = zeros(7*4,5);
for i=1:7
    RES((i-1)*4+1:i*4,1:5) = [Wm(i,:);Ws(i,:);Wq1(i,:);Wq2(i,:)];
end
