% LAT - Latvian DCE, 2017-01-03
% Latent Class Model with socio-demographics used as class membership variables

clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('LAT_data.mat');
DATA = load(EstimOpt.DataFile);
load('IndComplete.mat')


%% ****************************  data transformations  ****************************


DATA.SQ = DATA.ALT == 3;

% unique(DATA.RS) % reduced no. of native species
DATA.RS0 = DATA.RS == 0; % baseline, no change
DATA.RS1 = DATA.RS == 1; % minor improvement
DATA.RS2 = DATA.RS == 2; % major improvement

% unique(DATA.WQ) % water quality
DATA.WQ0 = DATA.WQ == 0; % baseline, no change
DATA.WQ1 = DATA.WQ == 1; % minor improvement
DATA.WQ2 = DATA.WQ == 2; % major improvement

% unique(DATA.IS) % invasive species
DATA.IS0 = DATA.IS == 0; % baseline, no change
DATA.IS1 = DATA.IS == 1; % minor improvement
DATA.IS2 = DATA.IS == 2; % major improvement

DATA.inc(DATA.Q51 == 1) = 50;
DATA.inc(DATA.Q51 == 2) = 75;
DATA.inc(DATA.Q51 == 3) = 125;
DATA.inc(DATA.Q51 == 4) = 175;
DATA.inc(DATA.Q51 == 5) = 225;
DATA.inc(DATA.Q51 == 6) = 275;
DATA.inc(DATA.Q51 == 7) = 325;
DATA.inc(DATA.Q51 == 8) = 475;
DATA.inc(DATA.Q51 == 9) = 450;
DATA.inc(DATA.Q51 == 10) = 500;
DATA.inc = DATA.inc'*1.5/100;
DATA.inc_nan = DATA.Q51 == 11;


%% ****************************  model specification  ****************************


DATA.Y = DATA.CHOICE;

% recode all choices of those who are not willing to pay to SQ.
% DATA.Q23: Q23 'Would you personally be willing to pay anything to improve the Latvian marine environment quality concerning the described problems?'
% 1 'Yes' 2 'No' 3 'Not sure / Hard to say'
DATA.Y(DATA.SQ == 1 & DATA.Q23 == 2) = 1;
DATA.Y(DATA.SQ ~= 1 & DATA.Q23 == 2) = 0;

DATA.X = [DATA.SQ, DATA.RS1, DATA.RS2, DATA.WQ1, DATA.WQ2, DATA.IS1, DATA.IS2, -DATA.COST];
EstimOpt.NamesA = {'Status quo';'RS 1';'RS 2';'WQ 1';'WQ 2';'IS 1';'IS 2';'-Cost'};

DATA.Xc = [DATA.Age, DATA.Q42==1, DATA.Q43==1, DATA.Q44, DATA.Q45, DATA.Q46==1, DATA.Q46==2, DATA.Q46==3, DATA.Q46==4,...
    DATA.Q47==2, DATA.Q47==3, DATA.Q47==4, DATA.Q47==5, DATA.Q47==6, DATA.Q47==7,...
    DATA.Q49_region==2, DATA.Q49_region==3, DATA.Q49_region==4, DATA.Q49_region==5, DATA.Q49_region==6, ...
    DATA.inc/100, DATA.inc_nan]; % Structural equations variables
EstimOpt.NamesC = {'age'; 'male'; 'Latvian'; 'HH size'; 'HH children'; 'edu 1'; 'edu 2'; 'edu 3'; 'edu 4'; ...
    'Occ part'; 'Occ retired'; 'Occ student'; 'Occ home'; 'Occ self'; 'Occ unemployed';...
    'region - Pier?ga'; 'region - Vidzeme'; 'region - Kurzeme'; 'region - Zemgale'; 'region - Latgale';...
    'income'; 'income missing'};


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter); %(~(DATA.Q23 == 2));%(~(DATA.Design == 1 & DATA.ChT10_Q33==2)); %(DATA.CT == 1); %DATA.liczba_alt == 2);
INPUT.Xa = DATA.X(DATA.filter,:); %(~(DATA.Q23 == 2),:);%(~(DATA.Design == 1 & DATA.ChT10_Q33==2),:); %(DATA.CT == 1,:); %(DATA.liczba_alt == 2,:);
INPUT.Xc = DATA.Xc(DATA.filter,:);
mXc = mean(INPUT.Xc);
sXc = std(INPUT.Xc);
INPUT.Xc = INPUT.Xc - mXc;
INPUT.Xc = INPUT.Xc ./ sXc;

MissingCompleteInd = MissingCompleteInd>0;
INPUT.MissingInd = MissingCompleteInd(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person
EstimOpt.NAlt = 3; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


EstimOpt.eps = 1.e-12; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1; % maximum number of iterations

% OptimOpt.Algorithm = 'trust-region'; %'quasi-newton'
% EstimOpt.NumGrad = 1; % 1
% OptimOpt.GradObj = 'off'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% EstimOpt.HessEstFix = 1 ; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian

%% **************************** MNL ****************************


EstimOpt.WTP_space = 1; % number of cost parameters for WTP-space estimation (need to come last in Xa)

% -14134.2860
B_backup = [11.4384004264175;0.609672548758381;-0.555254310971308;6.08647446725931;7.03327017187824;1.16720699691117;1.88112245241146;0.101049749474940];

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);



%% ****************************     LC     ****************************


EstimOpt.NClass = 6;

% -6229.8169
B_backup = [-5.81206170392991;2.81189552970775;1.01189541550570;44.8404136755663;57.6026118885196;13.1044140396443;13.7420038034547;0.0557236427139344;-10.1465688938600;-0.303012114138390;-1.09074925252985;1.33097830768363;0.928952441109129;1.12880430183148;0.602036467861480;0.469022997995663;13.3207409331422;0.729988334711496;1.54057812191349;3.32761865162797;1.78851196420608;0.337592569398418;-0.197552489742643;0.468787911957378;-0.715344520846458;1.01373352780708;0.567776368814822;2.00942685781585;2.73860343563196;0.396100513602616;-0.0617234828310164;0.350611236236133;-3.36645864655001;-0.312472534577357;-0.0650044552404968;0.687934558756737;0.596514703859046;0.0543176609209753;0.0507643605942281;1.96457710949013;56.8796626979001;-1.23446560470662;-1.21380570321491;-9.32433123650674;-6.06872383908469;-7.17957643879353;-8.30470022243128;-0.0446552818334418;-0.0103836751277208;-0.0431196478496742;-0.182402176126643;-0.278305059820453;-0.295779060519050;0.342668975631449;-1.61468848915474;-0.132017527184797;-0.0565018033848939;-0.182332873928098;-0.0471406301280516;-0.0429015241756577;0.0714924559517629;-0.0900003521768451;-0.293386128768802;0.216418965619208;0.124703212415850;0.0935758688397906;-0.202176529279063;0.181622495808533;0.220909637919029;0.147848340022354;-0.272160969128797;0.110728996187154;0.322321834116123;-0.246601264618622;-0.211768508159341;-0.301387361678732;0.190935335612281;-1.61948760216421;0.192687410399370;-0.0767231446548867;0.106233402821501;-0.108084540710481;-0.241715879573774;-0.147688281988075;-0.166273433192936;0.0690593511680376;-0.130899592864614;0.00321530006860793;0.0192840757117671;-0.0875583496293521;0.0593500583328617;0.283021742654586;-0.173960841773429;-0.394054023715133;1.90857240289926;0.687268929240508;-0.140351186912658;-0.454645090929511;-0.280368707635186;0.223079803256199;-0.122154894210363;0.0968148510798266;0.0107799562455594;-0.00695277607958126;0.0394272138528293;-0.0804371534076805;-0.147122339759295;-0.211364584256506;-0.0801333456201975;0.0274656185524363;-0.0227163963772875;0.222855602688923;-0.0398892834507899;0.156662691984147;0.488945839364944;-0.287219796755422;-0.270445575864623;-0.187075383235661;0.474083937031965;-0.322369151933187;-0.198014097131420;-0.349815070350773;0.418324729871585;-0.0465635037973157;0.376058440702868;0.215250486168298;0.338998167185932;-0.0522590730138493;-0.557422825948495;-0.0225708932127515;-0.199758397809243;-0.0935653405314070;-0.145465169782844;-0.148134944313352;-0.0116508462638845;-0.143140996326989;-0.206873765719549;0.358160600036504;-0.303054635848995;-0.253838154545593;-1.15154849241073;1.15768396879735;-0.116476923375131;-0.248028111298883;-0.468433248204638;0.558299791040488;-1.74449172366603;-0.260732546670705;0.168988036371292;-0.0133326806931567;0.305555734053659;-0.0498181290945357;-0.0473345132727733;0.0165056550125017;0.0280156927482541;0.0579229943484358;-0.0489389394473492;-0.121995655023919;-0.522081452030225;0.161214061124370;0.460340312008392;-0.0885835455356297;-0.480592435850617];

Results.LC = LC(INPUT,Results,EstimOpt,OptimOpt);
xlswrite('LC.xls', Results.LC.R_out) 


%% Simulate WTP of HH-types:


% model households:
HHtypes = ...
[20, 45, 35, 30, 70; ... % age
  0,  1,  1,  0,  0; ... % male
  1,  1,  0,  0,  1; ... % latvian
  1,  6,  2,  2,  1; ... % HHsize
  0,  4,  0,  1,  0; ... % children
  0,  0,  0,  0,  0; ... % edu primary
  0,  0,  0,  1,  1; ... % edu compulsory
  1,  0,  0,  0,  0; ... % edu general secondary
  0,  1,  0,  0,  0; ... % edu vocational secondary
  0,  0,  0,  0,  0; ... % 'Occ part'
  0,  0,  0,  0,  1; ... % 'Occ retired'
  1,  0,  0,  0,  0; ... % 'Occ student'
  0,  0,  0,  1,  0; ... % 'Occ home'
  0,  0,  1,  0,  0; ... % 'Occ self'
  0,  0,  0,  0,  0; ... % 'Occ unemployed'
  0,  0,  0,  1,  0; ... % 'region - Pier?ga'
  0,  1,  0,  0,  0; ... % 'region - Vidzeme'
  0,  0,  0,  0,  1; ... % 'region - Kurzeme'
  0,  0,  0,  0,  0; ... % 'region - Zemgale'
  0,  0,  0,  0,  0; ... % 'region - Latgale'
0.7,4.1,7.1,  0,2.6; ... % inc
  0,  0,  0,  1,  0];    % inc missing
HHtypes(21,:) = HHtypes(21,:)./100;

HHtypes = HHtypes - mXc';
HHtypes = HHtypes ./ sXc';
  
HHtypes = [ones(1,size(HHtypes,2));HHtypes];

sim1 = 1e4;
sim2 = 1e4;

bhat = mvnrnd(Results.LC.bhat,Results.LC.ihess,sim1)';
WTP_i = zeros(EstimOpt.NVarA,5,sim1);

NVarC = size(INPUT.Xc,2)+1;
parfor i = 1:sim1
    bhat_i = bhat(:,i);    
    bhat_i_Xc = reshape(bhat_i(EstimOpt.NVarA*EstimOpt.NClass+1:end),[1+size(INPUT.Xc,2),EstimOpt.NClass-1]);
    bhat_i_Xc = [bhat_i_Xc,zeros(size(bhat_i_Xc,1),1)];
    
    PClass = exp(HHtypes'*bhat_i_Xc);
    PClass_sum = sum(PClass,2);
    PClass = PClass./PClass_sum;

    bhat_i_Xa = reshape(bhat_i(1:EstimOpt.NVarA*EstimOpt.NClass),[EstimOpt.NVarA,EstimOpt.NClass]);
    WTP_i(:,:,i) = bhat_i_Xa * PClass';
end

Wm = median(WTP_i,3);
Ws = std(WTP_i,[],3);
Wq1 = quantile(WTP_i,0.025,3);
Wq2 = quantile(WTP_i,0.975,3);
% Wp = pv(Wm,Ws);

RES = zeros(7*4,5);
for i=1:7
    RES((i-1)*4+1:i*4,1:5) = [Wm(i,:);Ws(i,:);Wq1(i,:);Wq2(i,:)];
end
