% LAT - Latvian DCE, 2017-01-03
% Mixed Logit Model with socio-demographics used as interactions with random parameters means


clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('LAT_data.mat');
DATA = load(EstimOpt.DataFile);
load('IndComplete.mat');


%% ****************************  data transformations  ****************************


DATA.SQ = DATA.ALT == 3;

% unique(DATA.RS) % reduced no. of native species
DATA.RS0 = DATA.RS == 0; % baseline, no change
DATA.RS1 = DATA.RS == 1; % minor improvement
DATA.RS2 = DATA.RS == 2; % major improvement

% unique(DATA.WQ) % water quality
DATA.WQ0 = DATA.WQ == 0; % baseline, no change
DATA.WQ1 = DATA.WQ == 1; % minor improvement
DATA.WQ2 = DATA.WQ == 2; % major improvement

% unique(DATA.IS) % invasive species
DATA.IS0 = DATA.IS == 0; % baseline, no change
DATA.IS1 = DATA.IS == 1; % minor improvement
DATA.IS2 = DATA.IS == 2; % major improvement

DATA.inc(DATA.Q51 == 1) = 50;
DATA.inc(DATA.Q51 == 2) = 75;
DATA.inc(DATA.Q51 == 3) = 125;
DATA.inc(DATA.Q51 == 4) = 175;
DATA.inc(DATA.Q51 == 5) = 225;
DATA.inc(DATA.Q51 == 6) = 275;
DATA.inc(DATA.Q51 == 7) = 325;
DATA.inc(DATA.Q51 == 8) = 475;
DATA.inc(DATA.Q51 == 9) = 450;
DATA.inc(DATA.Q51 == 10) = 500;
DATA.inc = DATA.inc'*1.5/100;
DATA.inc_nan = DATA.Q51 == 11;


%% ****************************  model specification  ****************************


DATA.Y = DATA.CHOICE;

% recode all choices of those who are not willing to pay to SQ.
% DATA.Q23: Q23 'Would you personally be willing to pay anything to improve the Latvian marine environment quality concerning the described problems?'
% 1 'Yes' 2 'No' 3 'Not sure / Hard to say'
DATA.Y(DATA.SQ == 1 & DATA.Q23 == 2) = 1;
DATA.Y(DATA.SQ ~= 1 & DATA.Q23 == 2) = 0;

DATA.X = [DATA.SQ, DATA.RS1, DATA.RS2, DATA.WQ1, DATA.WQ2, DATA.IS1, DATA.IS2, -DATA.COST];
EstimOpt.NamesA = {'Status quo';'RS 1';'RS 2';'WQ 1';'WQ 2';'IS 1';'IS 2';'-Cost'};

DATA.Xm = [DATA.Age, DATA.Q42==1, DATA.Q43==1, DATA.Q44, DATA.Q45, DATA.Q46==1, DATA.Q46==2, DATA.Q46==3, DATA.Q46==4,...
    DATA.Q47==2, DATA.Q47==3, DATA.Q47==4, DATA.Q47==5, DATA.Q47==6, DATA.Q47==7,...
    DATA.Q49_region==2, DATA.Q49_region==3, DATA.Q49_region==4, DATA.Q49_region==5, DATA.Q49_region==6, ...
    DATA.inc/100, DATA.inc_nan]; % Structural equations variables
EstimOpt.NamesM = {'age'; 'male'; 'Latvian'; 'HH size'; 'HH children'; 'edu 1'; 'edu 2'; 'edu 3'; 'edu 4'; ...
    'Occ part'; 'Occ retired'; 'Occ student'; 'Occ home'; 'Occ self'; 'Occ unemployed';...
    'region - Pier?ga'; 'region - Vidzeme'; 'region - Kurzeme'; 'region - Zemgale'; 'region - Latgale';...
    'income'; 'income missing'};
% DATA.Xm = DATA.Xm ./ mean(DATA.Xm);

%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter); %(~(DATA.Q23 == 2));%(~(DATA.Design == 1 & DATA.ChT10_Q33==2)); %(DATA.CT == 1); %DATA.liczba_alt == 2);
INPUT.Xa = DATA.X(DATA.filter,:); %(~(DATA.Q23 == 2),:);%(~(DATA.Design == 1 & DATA.ChT10_Q33==2),:); %(DATA.CT == 1,:); %(DATA.liczba_alt == 2,:);
INPUT.Xm = DATA.Xm(DATA.filter,:);
mXm = mean(INPUT.Xm);
sXm = std(INPUT.Xm);
INPUT.Xm = INPUT.Xm - mXm;
INPUT.Xm = INPUT.Xm ./ sXm;

MissingCompleteInd = MissingCompleteInd>0;
INPUT.MissingInd = MissingCompleteInd(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person
EstimOpt.NAlt = 3; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


% EstimOpt.eps = 1.e-9; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1; % maximum number of iterations

% OptimOpt.Algorithm = 'trust-region'; %'quasi-newton'
% EstimOpt.NumGrad = 1; % 1
% OptimOpt.GradObj = 'off'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% EstimOpt.HessEstFix = 1 ; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian
EstimOpt.RealMin = 1;


%% **************************** MNL ****************************


EstimOpt.WTP_space = 1; % number of cost parameters for WTP-space estimation (need to come last in Xa)

% -13715.2412
B_backup = [12.7527113077509;0.710774202384766;-0.886471684554335;5.64632414271409;6.68551040165918;1.15212123716113;2.16310207498106;0.112232140858295;-1.21909248896991;-0.128588379965174;0.799976973193521;-1.80077486939550;-2.08358767586966;0.370917909642691;-0.0625609662824727;0.0415526792146938;0.113078285654487;0.0595848535937685;0.487952580079253;0.0347186582570583;0.608252476349727;-0.239408904214174;-0.00206708683870931;0.00110545432505216;-0.897308634016609;-0.0606989461749070;-0.205313256907795;-0.266324677429663;-0.728728169285188;0.292786083840089;0.521508435784986;-0.0116823291237417;1.08707379684571;0.0495574209171923;0.487621709372177;0.854508885352146;-0.0569583028074478;0.562590127466730;0.563486627828471;-0.0103498028088206;-0.284888155448600;0.484848316775053;0.720562299234281;-0.358783838773362;0.668378523613672;-0.307368312023301;0.148125334732236;0.00879978956678276;2.57944463656423;-0.999583435293439;-1.91124439624301;-1.76369506996590;0.249533000451360;-2.16837101426327;-1.29173281575360;-0.00811303782489773;0.376203963182501;-0.235519491323834;-0.442747475941721;-1.05185256654578;-1.60712315354099;0.0220123661897516;0.0360360919629646;-0.00298725798577863;-0.861966720543085;-0.251883005191396;0.261165302802221;-0.254860859370455;-0.228570294318169;-0.0224544235566808;-0.183673138496324;0.0120142775038555;-0.535282479260661;-0.383966708622265;-0.381278993524310;-0.901599210005052;-0.929335196461510;0.183202134809618;0.0566092604010697;0.00117196448256066;-0.348813204942414;0.262741976695460;0.280029734139098;-0.277661499989325;-0.571851334656587;0.127755828366076;0.140691495332391;0.0124279882398210;1.10843558736701;0.0369499489173523;-0.305913227928453;0.552040096026180;0.267919414118367;-0.617063278252376;0.0867024850339769;-0.00901863600307967;1.23479774850866;0.610397041232502;0.720939725542237;0.292302671199557;0.933360133034072;0.607281511251577;1.16298383099628;-0.00237634365264442;-1.17903257675626;-0.326697918689438;-0.264967321782066;0.169110824859879;-0.0741689751952285;-0.0387797843921550;0.205879360395645;0.00202850366588758;-2.04080890007439;-0.387533094523678;-0.0291535472336526;-0.905991720417518;-0.904018730082723;-0.249244041075000;-0.128399562836873;0.00453238603284467;0.819657177960521;-0.284861983925138;-0.484768268214714;0.947051927231646;0.556937018778412;0.459889860890220;0.398485710109590;-0.00148223645552060;-0.387026215346740;-0.332254953958810;-0.138284121041280;1.12525608127607;1.55001875938505;0.476932978433752;-0.140409160850340;0.00533967554675398;1.06389979389957;0.0222850805090399;-0.0561700953589336;-0.137969748672698;0.542579851307499;-0.313976007626856;-0.541452510760039;0.00691673880130704;-1.29832503764823;-0.769253194323607;-1.03459706837323;-0.153879369222492;-0.233265511157427;-0.353343610970450;-0.284921866911518;0.00630326112995528;-1.38835545664550;-0.628124816772729;-0.143954333403807;-0.193974604107264;0.0836788973701585;-0.125311116579616;-0.149743420563006;0.0172555472920677;-0.287599233064727;-0.782010711300824;-0.631993149921209;-0.642501100570252;-0.420825617417122;-0.314266817700383;-0.290459182770991;0.0131497311642796;1.69886322635822;-0.0423197955472965;0.0432042581351670;1.56871526954717;1.25305615140323;0.732738535776797;0.824984650066765;-0.0210309369104307;7.61782899642465;0.207808678177775;0.341954577916231;1.11138118613338;0.324835361044396;1.45782297518171;1.58957211674918;-0.0307068536976378];

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);



%% ****************************     MXL     ****************************


EstimOpt.Dist = [1 0 0 0 0 0 0 0 1];

% WTP-space: -6846.2577
B_backup = [17.4080319886705;0.604146802564857;0.602934127369904;4.43673608988790;5.49601881443731;1.21055123437164;0.592179052178513;-1.10001786665419;67.4693143494819;-0.165789611626176;0.377139872236372;1.54161277530880;3.79330336373693;-0.485054871049302;0.0769900651327838;1.08855588474471;17.5760427620060;-0.348660653988418;-0.333871725055929;-1.79694753772252;-2.43631616826259;-0.187658188499755;0.0738206838478839;0.366866820620073;0.701365640205551;-0.0335609513827634;0.121205186780185;0.0760007427784420;0.760767524712348;0.152681210631731;0.126854848923299;-0.0163948254265478;-7.56155703777516;0.183270628856743;0.211995532355284;-0.183492606691422;-0.456428454197877;0.0633941058793872;0.376965467530446;-0.158478677710648;-2.71091274133949;0.183574937194040;0.434213962031930;0.698038197082662;0.433406775325627;0.651514311568240;0.597539251184559;-0.0934511979083046;-2.41104325687465;0.0889142077269669;0.0230695867263797;-1.11326158940049;-0.883671241255501;-0.460201809220279;-0.0480327034275900;0.0251194131357433;5.96964599891345;0.756342510997933;1.68637718993788;0.506107734170399;-0.943517499432716;1.36265914635071;0.735003267407126;-0.117597267339700;0.458227858613126;-0.312492298427833;-0.362366229145227;-0.607236581840863;-1.30174243273567;-0.300699440455962;-0.274750915182722;-0.0414568301747335;1.72533633091672;-0.263911567841796;0.0168020534643960;-0.0284247107213923;-0.190037559609750;-0.251501973466364;-0.244413640782561;0.0613470412818844;0.939049096769190;-0.416359229331950;-0.401434731165920;-0.731685668868093;-0.873063387342628;-0.251042713546728;-0.242372608727897;-0.0153312275073182;1.05802122463222;-0.0206418099445809;-0.136935030454357;-0.403832856325540;-0.829487291563300;-0.227981822309640;-0.0402879803287187;0.0230156481945105;7.40961972308068;0.0963962043249774;-0.0556388317280077;0.332474049740807;0.312168553918393;-0.445191305132904;-0.304404719887082;0.0115735765871853;-1.82616648181287;0.186007868223508;0.0827700580733159;0.259320374661110;0.702333923712337;0.288761166599905;0.528434603720351;-0.107926011037438;-1.83337788167073;-0.165273657903174;-0.0304750584075088;0.0475908933184460;0.237993306671006;-0.214673744004095;-0.0327124013579051;-0.0681359447051472;0.501100898864405;-0.224727352603725;-0.275216214737848;-0.651338447333954;-1.03499492260651;-0.244967746094606;-0.0781089659502963;-0.00114658059990098;0.255862476828929;-0.115157787281245;-0.242122922080363;0.977014468875698;0.907063293110695;0.350255643012668;0.133629350205769;-0.103108185380508;0.475224572409049;-0.0908232137536859;0.0872647281974240;1.17708299202182;1.56365016589135;0.168658000614915;-0.350748758067369;-0.102390887928289;4.79672610450908;-0.288856242123189;-0.0638253623127034;0.353408078698750;0.833281171311286;-0.146052162643495;-0.231115273198029;0.0775167618017901;1.26280336287666;-0.261518124256285;-0.746081787213684;0.251569077710699;-0.110469596320594;-0.322984866024361;-0.464848760522348;-0.130487727015348;-0.0145354681508868;-0.128129337958134;-0.116266819739646;0.502096866826469;0.552544722906673;-0.0603164491812339;-0.183247174804351;0.0363060301016205;5.93533207365247;-0.428038831131517;-0.458048931773267;-0.430570848619769;-0.603217625744951;-0.277891075964866;-0.194992969736053;0.0776862123246105;-6.28106612250583;-0.150034293397720;0.0399097219655336;0.773971701495632;0.776043090435644;0.103109266567022;0.0938785858962825;-0.184366353554762;0.759662061741979;0.0373226967626585;0.224332248138610;0.569915312234575;0.0542335211757331;0.186419394703463;0.128438175961897;-0.187789880003350];

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% sim WTP:


HHtypes = ...
    [20, 45, 35, 30, 70; ... % age
    0,  1,  1,  0,  0; ... % male
    1,  1,  0,  0,  1; ... % latvian
    1,  6,  2,  2,  1; ... % HHsize
    0,  4,  0,  1,  0; ... % children
    0,  0,  0,  0,  0; ... % edu primary
    0,  0,  0,  1,  1; ... % edu compulsory
    1,  0,  0,  0,  0; ... % edu general secondary
    0,  1,  0,  0,  0; ... % edu vocational secondary
    0,  0,  0,  0,  0; ... % 'Occ part'
    0,  0,  0,  0,  1; ... % 'Occ retired'
    1,  0,  0,  0,  0; ... % 'Occ student'
    0,  0,  0,  1,  0; ... % 'Occ home'
    0,  0,  1,  0,  0; ... % 'Occ self'
    0,  0,  0,  0,  0; ... % 'Occ unemployed'
    0,  0,  0,  1,  0; ... % 'region - Pier?ga'
    0,  1,  0,  0,  0; ... % 'region - Vidzeme'
    0,  0,  0,  0,  1; ... % 'region - Kurzeme'
    0,  0,  0,  0,  0; ... % 'region - Zemgale'
    0,  0,  0,  0,  0; ... % 'region - Latgale'
    0.7,4.1,7.1,  0,2.6; ... % inc
    0,  0,  0,  1,  0];    % inc missing

HHtypes(21,:) = HHtypes(21,:)./100;

HHtypes = HHtypes - mXm';
HHtypes = HHtypes ./ sXm';

sim1 = 1e4;
sim2 = 1e4;
NVarA = EstimOpt.NVarA;

bhat = mvnrnd(Results.MXL_d.bhat,Results.MXL_d.ihess,sim1)';
WTP_i = zeros(sim1,NVarA,5);

parfor i = 1:sim1
    bhat_i = bhat(:,i);
    bhat_i_Xm = reshape(bhat_i(NVarA*2+1:end),[EstimOpt.NVarA,size(INPUT.Xm,2)]);
    B0_i = mvnrnd(bhat_i(1:NVarA)',diag(bhat_i(NVarA+1:NVarA*2).^2),sim2);
    B0_i = reshape(B0_i,[NVarA,1,sim2]) + bhat_i_Xm*HHtypes;
    WTP_i(i,:,:) = mean(B0_i,3);
end

Wm = squeeze(median(WTP_i));
Ws = squeeze(std(WTP_i));
Wq1 = squeeze(quantile(WTP_i,0.025));
Wq2 = squeeze(quantile(WTP_i,0.975));

RES = zeros(7*4,5);
for i=1:7
    RES((i-1)*4+1:i*4,1:5) = [Wm(i,:);Ws(i,:);Wq1(i,:);Wq2(i,:)];
end
