% LAT - Latvian DCE, 2017-01-03
% Mixed Logit Model with factor scores used as interactions with random parameters means

clear all
clc

global B_backup; % this one is for storing B in case iterations are interrupted with ctrl-c


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('LAT_data.mat');
DATA = load(EstimOpt.DataFile);
load('IndComplete.mat')


%% ****************************  data transformations  ****************************


DATA.SQ = DATA.ALT == 3;

% unique(DATA.RS) % reduced no. of native species
DATA.RS0 = DATA.RS == 0; % baseline, no change
DATA.RS1 = DATA.RS == 1; % minor improvement
DATA.RS2 = DATA.RS == 2; % major improvement

% unique(DATA.WQ) % water quality
DATA.WQ0 = DATA.WQ == 0; % baseline, no change
DATA.WQ1 = DATA.WQ == 1; % minor improvement
DATA.WQ2 = DATA.WQ == 2; % major improvement

% unique(DATA.IS) % invasive species
DATA.IS0 = DATA.IS == 0; % baseline, no change
DATA.IS1 = DATA.IS == 1; % minor improvement
DATA.IS2 = DATA.IS == 2; % major improvement

DATA.inc(DATA.Q51 == 1) = 50;
DATA.inc(DATA.Q51 == 2) = 75;
DATA.inc(DATA.Q51 == 3) = 125;
DATA.inc(DATA.Q51 == 4) = 175;
DATA.inc(DATA.Q51 == 5) = 225;
DATA.inc(DATA.Q51 == 6) = 275;
DATA.inc(DATA.Q51 == 7) = 325;
DATA.inc(DATA.Q51 == 8) = 475;
DATA.inc(DATA.Q51 == 9) = 450;
DATA.inc(DATA.Q51 == 10) = 500;
DATA.inc = DATA.inc'*1.5/100;
DATA.inc_nan = DATA.Q51 == 11;

%% Factor Analysis:


DATA.Xm = [DATA.Age, DATA.Q42==1, DATA.Q43==1, DATA.Q44, DATA.Q45, DATA.Q46==1, DATA.Q46==2, DATA.Q46==3, DATA.Q46==4,...
    DATA.Q47==2, DATA.Q47==3, DATA.Q47==4, DATA.Q47==5, DATA.Q47==6, DATA.Q47==7,...
    DATA.Q49_region==2, DATA.Q49_region==3, DATA.Q49_region==4, DATA.Q49_region==5, DATA.Q49_region==6, ...
    DATA.inc/100, DATA.inc_nan]; % Structural equations variables

DATA.Xm = DATA.Xm(1:3*12:end,:);
stdevF = std(DATA.Xm);
meanF = mean(DATA.Xm);

[lambda,psi,~,~,F] = factoran(DATA.Xm,6);

F = reshape(repmat(F',[3*12,1]),[6,3*12*1356])';


%% ****************************  model specification  ****************************


DATA.Y = DATA.CHOICE;

% recode all choices of those who are not willing to pay to SQ.
% DATA.Q23: Q23 'Would you personally be willing to pay anything to improve the Latvian marine environment quality concerning the described problems?'
% 1 'Yes' 2 'No' 3 'Not sure / Hard to say'
DATA.Y(DATA.SQ == 1 & DATA.Q23 == 2) = 1;
DATA.Y(DATA.SQ ~= 1 & DATA.Q23 == 2) = 0;

DATA.X = [DATA.SQ, DATA.RS1, DATA.RS2, DATA.WQ1, DATA.WQ2, DATA.IS1, DATA.IS2, -DATA.COST];
EstimOpt.NamesA = {'Status quo';'RS 1';'RS 2';'WQ 1';'WQ 2';'IS 1';'IS 2';'-Cost'};

DATA.Xm = F;
EstimOpt.NamesM = {'F1'; 'F2'; 'F3'; 'F4'; 'F5'; 'F6'};


%% ****************************  specifying input ****************************


DATA.filter = ones(size(DATA.Y))==1;

INPUT.Y = DATA.Y(DATA.filter); %(~(DATA.Q23 == 2));%(~(DATA.Design == 1 & DATA.ChT10_Q33==2)); %(DATA.CT == 1); %DATA.liczba_alt == 2);
INPUT.Xa = DATA.X(DATA.filter,:); %(~(DATA.Q23 == 2),:);%(~(DATA.Design == 1 & DATA.ChT10_Q33==2),:); %(DATA.CT == 1,:); %(DATA.liczba_alt == 2,:);
INPUT.Xm = DATA.Xm(DATA.filter,:);

MissingCompleteInd = MissingCompleteInd>0;
INPUT.MissingInd = MissingCompleteInd(DATA.filter,:);


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12; % Number of choice tasks per person
EstimOpt.NAlt = 3; % Number of alternatives
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt; % numel(unique(DATA.nr))


%% **************************** estimation and optimization options ****************************


% EstimOpt.eps = 1.e-9; % overall precision level

[INPUT, Results, EstimOpt, OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% OptimOpt.MaxIter = 1; % maximum number of iterations

% OptimOpt.Algorithm = 'trust-region'; %'quasi-newton'
% EstimOpt.NumGrad = 1; % 1
% OptimOpt.GradObj = 'off'; % 'off'
% OptimOpt.FinDiffType = 'central'; % 'forward'
% OptimOpt.Hessian = 'user-supplied'; % 'off'
% EstimOpt.HessEstFix = 1 ; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian


%% **************************** MNL ****************************


EstimOpt.WTP_space = 1; % number of cost parameters for WTP-space estimation (need to come last in Xa)

% -14134.2860
B_backup = [13.4464794594317;0.787438603105063;-0.668413039247077;6.47489844330521;7.16838965644303;1.78811446460011;2.42357218957629;0.111098009632852;0.850275544294011;0.374851757422323;0.602725505754468;1.03336657946889;1.50802282669062;0.169837972806034;0.549371418901399;-0.0141088899102025;10.7600562399523;0.649791626092250;0.315304464524073;2.27495181656263;1.43267180499067;2.03426757440494;1.73657381210976;-0.0238710155894962;-2.57356272961576;-0.258859775264722;0.0772291084843148;-2.09782390009686;-2.29797687236112;-0.922081097503242;-0.799382350878237;0.0382429863707609;-0.411752209177971;-0.0711050257327677;-0.240035626707138;-0.993266248150885;-1.43444712576456;-0.293311969576698;-0.175848827860835;0.00537831299687799;0.466068488596076;0.193803605977492;0.362479347508011;0.794710266301809;0.925585114853431;0.00606376196623509;0.0333577160240875;-0.00150470105030454;0.823451510977858;0.258197235134606;0.826731403613876;0.518225403955496;0.274432531486045;0.545897003851161;0.349107346773291;0.00105238820782786];

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     MXL     ****************************


EstimOpt.Dist = [1 0 0 0 0 0 0 0 1];

% WTP-space 10k: -6982.4354
B_backup = [18.2333600817485;0.616281229568389;0.506802942846493;3.85836392543662;4.56179981858080;0.922835952457063;0.514322683904366;-1.11876079043895;59.2562518799702;-0.118385603705262;-0.512931484362963;1.95024079049806;4.63326772216501;-0.247542990249385;-0.0891540252443061;1.00832468783986;-6.20807455627539;0.239226857116431;0.503677141533843;0.456265632844463;0.992050321681058;0.119915232308641;0.270104393858820;-0.257208103600627;2.42780120324646;0.0555448554734091;0.207046805750852;-0.0412874950782491;-0.548645952059218;0.0803362892845001;-0.0723849087541536;-0.0947521046352595;14.4470391726942;-0.137753423172862;-0.224282589671875;-1.38961662474441;-1.61013697469398;-0.457221686154030;-0.287328827180892;0.395752672753336;-0.385778932805090;-0.0237969165673229;-0.216736226236801;-0.725251104664522;-1.16232474833513;-0.183957106558167;-0.138053139631396;0.0360019833206282;-4.73579951520776;0.132409967038597;0.136539526021945;0.705736290615392;1.00698482765052;0.0114561072055975;-0.0143098666723316;-0.0618529086833826;-1.51298897390282;0.0691688642112645;0.269016208169120;0.244364625802962;-0.0226826426396497;0.153525526361827;0.170207314526496;-0.0203184259046347];

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt);


%% Simulate WTP of HH-types:

% model households:
HHtypes = ...
[20, 45, 35, 30, 70; ... % age
  0,  1,  1,  0,  0; ... % male
  1,  1,  0,  0,  1; ... % latvian
  1,  6,  2,  2,  1; ... % HHsize
  0,  4,  0,  1,  0; ... % children
  0,  0,  0,  0,  0; ... % edu primary
  0,  0,  0,  1,  1; ... % edu compulsory
  1,  0,  0,  0,  0; ... % edu general secondary
  0,  1,  0,  0,  0; ... % edu vocational secondary
  0,  0,  0,  0,  0; ... % 'Occ part'
  0,  0,  0,  0,  1; ... % 'Occ retired'
  1,  0,  0,  0,  0; ... % 'Occ student'
  0,  0,  0,  1,  0; ... % 'Occ home'
  0,  0,  1,  0,  0; ... % 'Occ self'
  0,  0,  0,  0,  0; ... % 'Occ unemployed'
  0,  0,  0,  1,  0; ... % 'region - Pier?ga'
  0,  1,  0,  0,  0; ... % 'region - Vidzeme'
  0,  0,  0,  0,  1; ... % 'region - Kurzeme'
  0,  0,  0,  0,  0; ... % 'region - Zemgale'
  0,  0,  0,  0,  0; ... % 'region - Latgale'
0.7,4.1,7.1,  0,2.6; ... % inc
  0,  0,  0,  1,  0];    % inc missing
HHtypes(21,:) = HHtypes(21,:)./100;

HHtypes = (HHtypes' - repmat(meanF,5,1)) ./ repmat(stdevF,5,1);
F3 = (HHtypes*diag(1./sqrt(psi))) / (lambda'*diag(1./sqrt(psi)));
  
sim1 = 1e4;
sim2 = 1e4;
NVarA = 8;

bhat = mvnrnd(Results.MXL_d.bhat,Results.MXL_d.ihess,sim1)';
WTP_i = zeros(sim1,(NVarA),5);

parfor i = 1:sim1
    bhat_i = bhat(:,i);
    bhat_i_Xm = reshape(bhat_i(NVarA*2+1:end),[EstimOpt.NVarA,size(INPUT.Xm,2)]);
    B0_i = mvnrnd(bhat_i(1:NVarA)',diag(bhat_i(NVarA+1:NVarA*2).^2),sim2);
    B0_i = reshape(B0_i,[NVarA,1,sim2]) + bhat_i_Xm*F3';
    WTP_i(i,:,:) = mean(B0_i,3);   
end

Wm = squeeze(median(WTP_i));
Ws = squeeze(std(WTP_i));
Wq1 = squeeze(quantile(WTP_i,0.025));
Wq2 = squeeze(quantile(WTP_i,0.975));

RES = zeros(7*4,5);
for i=1:7
    RES((i-1)*4+1:i*4,1:5) = [Wm(i,:);Ws(i,:);Wq1(i,:);Wq2(i,:)];
end
